'use strict';

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = _postcss2.default.plugin('postcss-selector-namespace', function () {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var _options$namespace = options.namespace;
  var namespace = _options$namespace === undefined ? '.self' : _options$namespace;
  var _options$selfSelector = options.selfSelector;
  var selfSelector = _options$selfSelector === undefined ? /:--namespace/ : _options$selfSelector;
  var _options$rootSelector = options.rootSelector;
  var rootSelector = _options$rootSelector === undefined ? /:root/ : _options$rootSelector;
  var _options$ignoreRoot = options.ignoreRoot;
  var ignoreRoot = _options$ignoreRoot === undefined ? true : _options$ignoreRoot;
  var _options$dropRoot = options.dropRoot;
  var dropRoot = _options$dropRoot === undefined ? true : _options$dropRoot;

  selfSelector = regexpToGlobalRegexp(selfSelector);

  return function (css, result) {
    css.walkRules(function (rule) {
      if (rule.parent.type === 'rule') {
        return;
      }

      rule.selectors = rule.selectors.map(function (selector) {
        selfSelector.lastIndex = 0;
        if (selfSelector.test(selector)) {
          return selector.replace(selfSelector, namespace);
        }

        if (ignoreRoot && selector.search(rootSelector) === 0) {
          if (dropRoot) {
            return selector.replace(rootSelector, '').trim();
          }

          return selector;
        }

        return namespace + ' ' + selector;
      });
    });
  };
});

/**
 * Newer javascript engines allow setting flags when passing existing regexp
 * to the RegExp constructor, until then, we extract the regexp source and
 * build a new object.
 *
 * @param {RegExp|string} regexp The regexp to modify
 * @return {RegExp} The new regexp instance
 */
function regexpToGlobalRegexp(regexp) {
  var source = regexp instanceof RegExp ? regexp.source : regexp;
  return new RegExp(source, 'g');
}